###############################################################################
#  Prioritization prep and analysis for butterfly SDMs
#  September 24, 2018; last updated November 13, 2018
#  Sara Williams
###############################################################################

# =============================================================================
#  Load packages and functions.
# =============================================================================
library(sf)
library(sp)
library(raster)
library(dplyr)
library(stringr)
library(ggplot2)
library(prioritizr)
library(gurobi)
library(rasterVis)
library(tidyr)


	
# =============================================================================
#  Load data.
# =============================================================================		

	# ----------------------
	#  Desired CRS.
	proj_UTM <- "+proj=utm +zone=50 +datum=WGS84 +units=m +no_defs +ellps=WGS84 +towgs84=0,0,0"

	# ----------------------
	#  Endemic butterfly ranges.
	#setwd("C:/Users/saraw/Documents/BiotropicaSDMPaper/Endemic_15") # Tif files found in 'InputMaps_1kmSabah' folder for 15 endemic species ('Endemic_15' sub-folder)
	end_fly_ls <- list.files(pattern='\\.tif$')
	end_fly_stack <- stack(end_fly_ls)
	
	# ----------------------	
	#  Non-endemic butterfly ranges.
	#setwd("C:/Users/saraw/Documents/BiotropicaSDMPaper/NonEndemic_62") # Tif files found in 'InputMaps_1kmSabah' folder for 62 non-endemic (Sundaland) species ('NonEndemic_62' sub-folder)
	non_end_fly_ls <- list.files(pattern='\\.tif$')
	non_end_fly_stack <- stack(non_end_fly_ls)
	
	#setwd("C:/Users/saraw/Documents/")
	#setwd("C:/Users/saraw/Documents/")

	# ----------------------
	#  All Sabah and mainland boundaries and TPAs (nb: spatial files not provided in archived dataset)
	load("Prioritization/study_area_boundaries_and_pas/sabah_border.Rdata") # Sabah boarder shapefile 
	load("Prioritization/study_area_boundaries_and_pas/main_sabah_sf.Rdata") # Mainland Sabah boarder shapefile (i.e., excluding offshore islands)
	load("Prioritization/study_area_boundaries_and_pas/sabah_tpa.Rdata") # Totally Protected Area (TPA) shapefiles; data obtained from Sabah Forestry Department 
	
	# ----------------------
	#  Forest area
	load(file = "Prioritization/study_area_cover/acd_agg_sf_for.Rdata") # Forest cover raster; aggregated 1 km grid-cells with >40 metric tons of aboveground carbon per hectare (Mg C per ha) (mean across 1 km grid-cells resampled from 30 m grid-cells) 
									    # Data for aboveground carbon was obtained from Asner et al. (2018) via the Sabah Forestry Department

	
# =============================================================================
#  Planning unit grid as raster across forested area only - include existing TPAs
# =============================================================================		

	# ----------------------
	#  Crop forest cover raster to mainland area only and convert to spatial object
	sabah_for <- st_intersection(acd_agg_sf_for, main_sabah_sf) 
	sabah_for_w_pa_sp <- as(sabah_for, 'Spatial')
	
	# ----------------------
	#  Make a study area raster template using a butterfly SDM layer
	r_template <- end_fly_stack[[1]]
	r_template[r_template > 0 ] <- 0
	
	# ----------------------
	#  Mask template by area that is covered to forest
	pu_r <- raster::mask(r_template, sabah_for_w_pa_sp)
	
	# ----------------------
	#  Set value to planning unit (each raster cell) to equal 100 ha
	#   1000 m x 1000 m = 100 ha
	pu_r[pu_r == 0] <- 100
	
	

# =============================================================================
#  Determine area covered by existing TPA network
# =============================================================================		

	# ----------------------
	#  Use only mainland Sabah TPAs
	main_sabah_tpa <- st_intersection(sabah_tpa, main_sabah_sf)
	
	# ----------------------
	#  Convert Sabah TPA sf objects into sp object
	main_sabah_tpa_sp <- as(main_sabah_tpa, "Spatial")
	main_sabah_sp <- as(main_sabah_sf, "Spatial")
	
	# ----------------------
	#  Mask template by area that is covered to forest
	tpa_r <- raster::mask(r_template, main_sabah_tpa_sp)
	tpa_r[tpa_r == 0] <- 100
	
	# ----------------------
	# Area of all TPA in ha = sum of cells
	tpa_area_ha <- cellStats(tpa_r, sum)
	
	# ----------------------
	# Test areas in TPA that are not forested
	tmp_pu_r <- pu_r
	tmp_pu_r[is.na(tmp_pu_r)] <- 0
	tmp_tpa_r <- tpa_r
	tmp_tpa_r[is.na(tmp_tpa_r)] <- 0
	tmp_tpa_r[tmp_tpa_r == 100] <- 500
	pu_tpa_sum <- sum(stack(tmp_pu_r, tmp_tpa_r))
	# 100 = planning unit only
	# 500 = tpa only
	# 600 = planning unit + tpa
	tpa_without_for <- pu_tpa_sum
	tpa_without_for[tpa_without_for != 500] <- 0
	tpa_without_for[tpa_without_for == 500] <- 100
	
	# ----------------------
	# Area of TPA without forest in ha = sum of cells
	tpa_without_for_area_ha <- cellStats(tpa_without_for, sum)

	
	
# =============================================================================
# Study area set up plots
# =============================================================================		

	
	## Key
	myColors <- c('palegreen4',  'dodgerblue4', rgb(0, 0, 0, 0.6))
	myKey <- list(text=list(lab = c("Forested Area", "Selected Units", "Existing Protected Area")),
				  rectangles=list(col = myColors, border = FALSE),
				  space='bottom', columns=3)
	main_sabah_sp <- as(main_sabah_sf, "Spatial")
	tpa_without_for[tpa_without_for ==0] <- NA
		
	p0 <- levelplot(pu_r, margin = FALSE, col.regions=c('palegreen4'))
	p1 <- levelplot(tpa_r, margin = FALSE, col.regions=c('goldenrod3'))
	p1 + as.layer(p0, under = TRUE) 
	
	p2 <- levelplot(tpa_without_for, margin = FALSE, col.regions=c('turquoise'))
	p2 + as.layer(p1, under = TRUE) + as.layer(p0, under = TRUE)
	
	

# =============================================================================
#  Run prioritzation for Borneo only endemics
# =============================================================================	
	
	# ----------------------
	#  Set up problem for scenario 1
	p_end <- problem(x = pu_r, features = end_fly_stack) %>%
		add_max_features_objective(tpa_area_ha) %>%
		add_relative_targets(0.5) %>%
		add_binary_decisions() %>%
		add_gurobi_solver()
	
	# ----------------------
	#  Solve.
	s_end <- solve(p_end)
	
	# ----------------------
	#  Set up problem for scenario 1 with BLM
	p_end_b <- problem(x = pu_r, features = end_fly_stack) %>%
		add_max_features_objective(tpa_area_ha) %>%
		add_relative_targets(0.5) %>%
		add_boundary_penalties(0.0000000001, 0.5) %>%
		add_binary_decisions() %>%
		add_gurobi_solver()
	
	# ----------------------
	#  Solve.
	s_end_b <- solve(p_end_b)

	# ----------------------
	#  Set up locked in and locked out areas for round 2.
	locked_in_new <- s_end_b
	
	# ----------------------
	#  Set up problem for scenario 1
	p_end_b_2 <- problem(x = pu_r, features = end_fly_stack) %>%
		add_max_utility_objective(tpa_area_ha) %>%
		add_locked_in_constraints(locked_in_new) %>%
		add_binary_decisions() %>%
		add_gurobi_solver()
	
	# ----------------------
	#  Solve.
	s_end_b_2 <- solve(p_end_b_2)
	
	#writeRaster(s_end, "s_end.tif")         # 'PrioritizedArea_EndemicSpecies.tif' file in 'Prioritization_OutputMaps' folder
	#writeRaster(s_end_b_2, "s_end_b.tif")   # 'PrioritizedArea_EndemicSpeciesBLM.tif' file in 'Prioritization_OutputMaps' folder
	
	
	
# =============================================================================
#  Run prioritzation for Sundaland endemics
# =============================================================================	
	
	# ----------------------
	#  Set up problem for scenario 1
	p_non_end <- problem(x = pu_r, features = non_end_fly_stack) %>%
		add_max_features_objective(tpa_area_ha) %>%
		add_relative_targets(0.4) %>%
		add_binary_decisions() %>%
		add_gurobi_solver()
	
	# ----------------------
	#  Solve.
	s_non_end <- solve(p_non_end)
	
	# ----------------------
	#  Set up locked in and locked out areas for round 2.
	locked_in_new <- s_non_end
	
	# ----------------------
	#  Set up problem for scenario 1
	p_non_end_2 <- problem(x = pu_r, features = non_end_fly_stack) %>%
		add_max_utility_objective(tpa_area_ha) %>%
		add_locked_in_constraints(locked_in_new) %>%
		add_binary_decisions() %>%
		add_gurobi_solver()
	
	# ----------------------
	#  Solve.
	s_non_end_2 <- solve(p_non_end_2)
	
	
	# ----------------------
	#  Set up problem for scenario 1 with BLM
	p_non_end_b <- problem(x = pu_r, features = non_end_fly_stack) %>%
		add_max_features_objective(tpa_area_ha) %>%
		add_relative_targets(0.4) %>%
		add_boundary_penalties(0.000000001, 0.5) %>%
		add_binary_decisions() %>%
		add_gurobi_solver()
	
	# ----------------------
	#  Solve.
	s_non_end_b <- solve(p_non_end_b)
	
	#writeRaster(s_non_end_2, "s_non_end.tif")
	#writeRaster(s_non_end_b, "s_non_end_b.tif")
	
	

# =============================================================================
#  Run prioritzation for all species
# =============================================================================	
	
	# ----------------------
	#  Combine endemic and non-endemic SDM stacks
	all_fly_stack <- stack(end_fly_stack, non_end_fly_stack)
	
	# ----------------------
	#  Set up problem for scenario 1
	p_all <- problem(x = pu_r, features = all_fly_stack) %>%
		add_max_features_objective(tpa_area_ha) %>%
		add_relative_targets(0.4) %>%
		add_binary_decisions() %>%
		add_gurobi_solver()
	
	# ----------------------
	#  Solve.
	s_all <- solve(p_all)
	
	# ----------------------
	#  Set up problem for scenario 1 with BLM
	p_all_b <- problem(x = pu_r, features = all_fly_stack) %>%
		add_max_features_objective(tpa_area_ha) %>%
		add_relative_targets(0.4) %>%
		add_boundary_penalties(0.0000000005, 0.5) %>%
		add_binary_decisions() %>%
		add_gurobi_solver()
	
	# ----------------------
	#  Solve.
	s_all_b <- solve(p_all_b)
	
	#writeRaster(s_all, "s_all.tif")       # 'PrioritizedArea_AllSpecies.tif' file in 'Prioritization_OutputMaps' folder
	#writeRaster(s_all_b, "s_all_b.tif")   # 'PrioritizedArea_AllSpeciesBLM.tif' file in 'Prioritization_OutputMaps' folder
	
	
	
	
# =============================================================================
#  Plot outputs.
# =============================================================================	
	
	# ----------------------
	#  Reload if needed
	# setwd("C:/Users/saraw/Documents/BiotropicaSDMPaper/Solutions")
	# s_end <- raster("s_end.tif")
	# s_end_b <- raster("s_end_b.tif")
	# s_non_end <- raster("s_non_end.tif")
	# s_non_end_b <- raster("s_non_end_b.tif")
	# s_all <- raster("s_all.tif")
	# s_all_b <- raster("s_all_b.tif")
	
	# ----------------------
	#  Stack and save all outputs
	s_stack <- stack(s_end, s_non_end, s_all, s_end_b, s_non_end_b, s_all_b)
	names(s_stack) <- c("endemics", "non_endemics", "all", 
		"endemics_blm", "non_endemics_blm",  "all_blm")
	#writeRaster(s_stack, file = "all_solutions.tif")

	# ----------------------
	#  Plot
	myColors <- c('palegreen4',  'dodgerblue4', rgb(0, 0, 0, 0.6))
	myKey <- list(text=list(lab = c("Forested Area", "Selected Units", "Existing Protected Area")),
		  rectangles=list(col = myColors, border = FALSE),
		  space='bottom', columns=3)
		
	p0 <- levelplot(pu_r, margin = FALSE, col.regions=c('palegreen4'))
	p1 <- levelplot(s_stack, layout=c(3, 2), margin = FALSE, col.regions=c('transparent', 'dodgerblue4'),
		xlab="", ylab="", scales=list(draw=FALSE), colorkey=FALSE, key = myKey,
		names.attr = c("Borneo Endemics (n = 15)", "Sundaland Endemics (n = 62)", "All Species (n = 77)" ,
		"Borneo Endemics with BLM", "Sundaland Endemics with BLM", "All Species with BLM"), 
		par.settings=list(layout.heights=list(key.bottom=2,key.top=1)))
	p1 + as.layer(p0, under = TRUE) 	+ 
		layer(sp.polygons(main_sabah_sp, lwd=0.8, col='black'), par.settings = list(superpose.line = list(lwd=2))) + 
		layer(sp.polygons(main_sabah_tpa_sp, lwd=0.8, col = "grey10", fill = "grey10", alpha = 0.6, 
			par.settings = list(superpose.line = list(lwd=3))))
	

	
# =============================================================================
#  Assess coverage of each raw SDM input by each solution and existing TPA network
# =============================================================================	
	
	# Only all species outputs
	s_stack <- stack(s_all, s_all_b)
	names(s_stack) <- c("all", "all_blm")
	
	# ----------------------
	#  Determine coverage of all species by solutions' selected planning units
	prop_cov_all_fly_sol <- matrix(NA, nrow = nlayers(all_fly_stack), ncol = nlayers(s_stack))
	
	for(j in 1:nlayers(s_stack)){
	
		# ----------------------
		#   Maskbutterfly SDMs by the solution.
		sol <- s_stack[[j]]
		sol[sol > 1] <- 1
		sol[is.na(sol)] <- 0
		all_fly_feat_m <- raster::mask(all_fly_stack, sol, maskvalue = 1, updatevalue = 0, inverse = TRUE)
	
		for(i in 1:nlayers(all_fly_stack)){
			range_cov_feat_sum <- cellStats(all_fly_stack[[i]], sum, na.rm = TRUE)
			range_cov_sol_sum <- cellStats(all_fly_feat_m[[i]], sum, na.rm = TRUE)
			cov <- range_cov_sol_sum/range_cov_feat_sum
			prop_cov_all_fly_sol[i, j] <- cov
			}
		}

	# ----------------------
	#  Determine coverage of all species by existing TPA network
	prop_cov_all_fly_tpa <- rep(NA, nlayers(all_fly_stack))
	
	# ----------------------
	#   Mask physographic provinces rasters by the solution.
	sol <- tpa_r
	sol[sol > 1] <- 1
	sol[is.na(sol)] <- 0
	all_fly_feat_m <- raster::mask(all_fly_stack, sol, maskvalue = 1, updatevalue = 0, inverse = TRUE)
	
	for(i in 1:nlayers(all_fly_stack)){
		range_cov_feat_sum <- cellStats(all_fly_stack[[i]], sum, na.rm = TRUE)
		range_cov_sol_sum <- cellStats(all_fly_feat_m[[i]], sum, na.rm = TRUE)
		cov <- range_cov_sol_sum/range_cov_feat_sum
		prop_cov_all_fly_tpa[i] <- cov
		}
		
	end_rep <- rep("Endemic", 15)
	non_end_rep <- rep("Non-endemic", 62)
	species_type <- c(end_rep, non_end_rep)
	
	all_coverage <- bind_cols(as.data.frame(prop_cov_all_fly_sol), as.data.frame(prop_cov_all_fly_tpa),
		as.data.frame(species_type))
	colnames(all_coverage) <- c("endemics_sol",  "non_endemics_sol", "all_sol", 
		"endemics_blm_sol", "non_endemics_blm_sol",  "all_blm_sol", "existing_tpa", "species_type")
	
	all_coverage_long <- all_coverage %>% gather(Solution, Proportion, -species_type)
	all_coverage_long$Solution <- factor(all_coverage_long$Solution, 
		levels = c("endemics_sol",  "non_endemics_sol", "all_sol", "endemics_blm_sol", 
			"non_endemics_blm_sol",  "all_blm_sol", "existing_tpa"))
	all_coverage_long$Proportion <- as.numeric(all_coverage_long$Proportion)
	
	mean_cov <- all_coverage_long %>%
		group_by(species_type, Solution) %>%
		mutate(mean_cov = mean(Proportion)) %>%
		slice(1) %>%
		as.data.frame()

	end_means <- round(colMeans(all_coverage[1:15, 1:7]), 2)
	non_end_means <- round(colMeans(all_coverage[16:77, 1:7]), 2)
	all_means <- round(colMeans(all_coverage[, 1:7]), 2)
	cov_means_df <- as.data.frame(rbind(end_means, non_end_means, all_means))

	# ----------------------
	# Box plot - for current scenario
	cov_p <- ggplot(all_coverage_long) +
		geom_violin(aes(x = Solution, y = Proportion, colour = Solution, fill = Solution), alpha = 0.8) +
		scale_fill_manual(name = "Solution",
			values = c("darkslategray1", "darkolivegreen1", "goldenrod1", 
				"darkslategray4", "darkolivegreen4", "goldenrod4", "orangered3"),
			labels = c("Endemics \nOnly",  "Non-endemics \nOnly", "All Species", 
				"BLM - Endemics \nOnly",  "BLM - Non-endemics \nOnly", "BLM - All Species", 
				"Existing TPA \nNetwork")) +
		scale_colour_manual(name = "", guide = FALSE,
			values = c("darkslategray1", "darkolivegreen1", "goldenrod2", 
				"darkslategray4", "darkolivegreen3", "goldenrod4", "orangered3")) +
		 scale_x_discrete(labels = c("EO",  "NO", "ALL", "EO \nBLM",  "NO \nBLM", "ALL \nBLM", "TPA")) +
		geom_point(data = mean_cov, aes(x = Solution, y = mean_cov, shape = species_type), size = 2) +
		scale_shape_manual(name = "Species Type", values = c(8, 19)) +
		xlab(" ") +
		ylab("Proportion of SDM Protected \n") +
		ylim(0.15, 0.6) +
		theme_bw() +
		theme(axis.text.x = element_text(face = "bold"),
			legend.key.size = unit(1, 'cm')) 
			#geom_hline(yintercept = 0.05, linetype="dashed", color = "grey80") 
	cov_p 




# =============================================================================
#  Other calculations
# =============================================================================	
	
	# ----------------------
	#  Overlap between TPA network and each solution
	tpa_r_tmp <- tpa_r
	tpa_r_tmp[tpa_r_tmp > 0] <- 5
	tpa_r_tmp[is.na(tpa_r_tmp)] <- 0
	
	tpa_sol_overlap <- as.numeric(rep(NA, nlayers(s_stack)))
	
	for(i in 1:nlayers(s_stack)){
		sol <- s_stack[[i]]
		
		tpa_sol <- stack(sol, tpa_r_tmp)
		tpa_sol_sum <- sum(tpa_sol)
		# 0 = Neither
		# 1 = Solution only
		# 5 = TPA only
		# 6 = Solution and TPA
		
		tpa_sol_sum[tpa_sol_sum < 6] <- 0
		tpa_sol_sum[tpa_sol_sum > 0] <- 100
		overlap_ha <- cellStats(tpa_sol_sum, sum)
		
		tpa_sol_overlap[i] <- overlap_ha
		}
	
	
	# ----------------------
	#  How many legislative TPAs do not have a single cell of prioritized area
	tpa_no_sol_mat <- matrix(NA, nrow = 248, ncol = nlayers(s_stack))
	tpa_no_sol_total <- as.numeric(rep(NA, nlayers(s_stack)))
	
	for(i in 1:nlayers(s_stack)){
		sol_tpa_ext <- raster::extract(s_stack[[i]], main_sabah_tpa_sp)
		sol_tpa_ext_sum <- sapply(sol_tpa_ext , sum, na.rm = TRUE)
		tpa_no_sol_mat[,i] <- as.vector(sol_tpa_ext_sum)
		count_tpa_no_sol <- length(which(sol_tpa_ext_sum < 1))
		tpa_no_sol_total[i] <- count_tpa_no_sol
		}
	
	tpa_no_sol_mat[tpa_no_sol_mat > 0] <- 1
	tpas_with_no_prior <- as.data.frame(tpa_no_sol_mat) %>%
		mutate(missing_from_all_tmp = nlayers(s_stack) - rowSums(.)) %>%
		mutate(missing_from_all = ifelse(missing_from_all_tmp > 0, 1, 0))
	
	tpa_no_sol_df <- as.data.frame(tpas_with_no_prior$missing_from_all_tmp)
	colnames(tpa_no_sol_df) <- "missing_from_sol"
	tpa_no_sol_sf <- bind_cols(main_sabah_tpa, tpa_no_sol_df) %>%
		dplyr::select(val = missing_from_sol)
	tpa_no_sol_sp <- as(tpa_no_sol_sf, "Spatial")
	#shapefile(tpa_no_sol_sp, file = "tpa_num_prioritizations_missing.shp")
	
	tpa_no_sol_p <- ggplot() +
		geom_sf(data = main_sabah_sf, colour = "grey50", fill = "grey50", alpha = 0.7) +
		geom_sf(data = p_sf, aes(fill = val), colour = "black") +
		scale_fill_gradient(high = "red", low ="white",
			name = "Number of prioritization \nscenarios that include \nno area in TPA") +
		#scale_colour_gradient(high = "red", low ="white") +
		coord_sf(crs = st_crs(32650)) +
		xlim(315000, 755000) +
		ylim(455000, 815000) +
		theme_bw()
	tpa_no_sol_p
	
	
	# ----------------------
	#  Overlap between all 3 scenarios (without BLM and with)
	s_stack[is.na(s_stack)] <- 0
	
	sum_s_stack_no_blm <- sum(s_stack[[1:3]])
	sum_s_stack_no_blm[sum_s_stack_no_blm < 3] <- 0
	sum_s_stack_no_blm[sum_s_stack_no_blm > 0] <- 100
	overlap_no_blm <- cellStats(sum_s_stack_no_blm, sum) 
	
	sum_s_stack_blm <- sum(s_stack[[4:6]])
	sum_s_stack_blm[sum_s_stack_blm < 3] <- 0
	sum_s_stack_blm[sum_s_stack_blm > 0] <- 100
	overlap_blm <- cellStats(sum_s_stack_blm, sum) 

	sum_s_stack_all <- sum(s_stack[[1:6]])
	sum_s_stack_all[sum_s_stack_all < 6] <- 0
	sum_s_stack_all[sum_s_stack_all > 0] <- 100
	overlap_all <- cellStats(sum_s_stack_all, sum) 	
	
	mypalette <- brewer.pal(8,"YlGnBu")
	p3 <- levelplot(sum_s_stack_all, margin = FALSE, col.regions = rep(mypalette, each = 2))
	p3 + layer(sp.polygons(main_sabah_sp, lwd=1, col='black'), par.settings = list(superpose.line = list(lwd=2))) + 
		layer(sp.polygons(main_sabah_tpa_sp, lwd=1, col = "grey10", fill = "transparent",  
			par.settings = list(superpose.line = list(lwd=3))))
	

